from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.user_docs_route_list_direction import UserDocsRouteListDirection
from ...models.user_docs_route_list_response_200 import UserDocsRouteListResponse200
from ...models.user_docs_route_list_sort_by import UserDocsRouteListSortBy
from ...types import UNSET, Response


def _get_kwargs(
    id: str,
    *,
    limit: int,
    offset: int,
    sort_by: UserDocsRouteListSortBy = UserDocsRouteListSortBy.CREATED_AT,
    direction: UserDocsRouteListDirection = UserDocsRouteListDirection.ASC,
    metadata_filter: str = "{}",
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["limit"] = limit

    params["offset"] = offset

    json_sort_by = sort_by.value
    params["sort_by"] = json_sort_by

    json_direction = direction.value
    params["direction"] = json_direction

    params["metadata_filter"] = metadata_filter

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/users/{id}/docs".format(
            id=id,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[UserDocsRouteListResponse200]:
    if response.status_code == HTTPStatus.OK:
        response_200 = UserDocsRouteListResponse200.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[UserDocsRouteListResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    limit: int,
    offset: int,
    sort_by: UserDocsRouteListSortBy = UserDocsRouteListSortBy.CREATED_AT,
    direction: UserDocsRouteListDirection = UserDocsRouteListDirection.ASC,
    metadata_filter: str = "{}",
) -> Response[UserDocsRouteListResponse200]:
    """List Docs owned by a User

    Args:
        id (str):
        limit (int): Limit the number of results
        offset (int): Offset to apply to the results
        sort_by (UserDocsRouteListSortBy):  Default: UserDocsRouteListSortBy.CREATED_AT.
        direction (UserDocsRouteListDirection):  Default: UserDocsRouteListDirection.ASC.
        metadata_filter (str):  Default: '{}'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[UserDocsRouteListResponse200]
    """

    kwargs = _get_kwargs(
        id=id,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        direction=direction,
        metadata_filter=metadata_filter,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    limit: int,
    offset: int,
    sort_by: UserDocsRouteListSortBy = UserDocsRouteListSortBy.CREATED_AT,
    direction: UserDocsRouteListDirection = UserDocsRouteListDirection.ASC,
    metadata_filter: str = "{}",
) -> Optional[UserDocsRouteListResponse200]:
    """List Docs owned by a User

    Args:
        id (str):
        limit (int): Limit the number of results
        offset (int): Offset to apply to the results
        sort_by (UserDocsRouteListSortBy):  Default: UserDocsRouteListSortBy.CREATED_AT.
        direction (UserDocsRouteListDirection):  Default: UserDocsRouteListDirection.ASC.
        metadata_filter (str):  Default: '{}'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        UserDocsRouteListResponse200
    """

    return sync_detailed(
        id=id,
        client=client,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        direction=direction,
        metadata_filter=metadata_filter,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    limit: int,
    offset: int,
    sort_by: UserDocsRouteListSortBy = UserDocsRouteListSortBy.CREATED_AT,
    direction: UserDocsRouteListDirection = UserDocsRouteListDirection.ASC,
    metadata_filter: str = "{}",
) -> Response[UserDocsRouteListResponse200]:
    """List Docs owned by a User

    Args:
        id (str):
        limit (int): Limit the number of results
        offset (int): Offset to apply to the results
        sort_by (UserDocsRouteListSortBy):  Default: UserDocsRouteListSortBy.CREATED_AT.
        direction (UserDocsRouteListDirection):  Default: UserDocsRouteListDirection.ASC.
        metadata_filter (str):  Default: '{}'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[UserDocsRouteListResponse200]
    """

    kwargs = _get_kwargs(
        id=id,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        direction=direction,
        metadata_filter=metadata_filter,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    limit: int,
    offset: int,
    sort_by: UserDocsRouteListSortBy = UserDocsRouteListSortBy.CREATED_AT,
    direction: UserDocsRouteListDirection = UserDocsRouteListDirection.ASC,
    metadata_filter: str = "{}",
) -> Optional[UserDocsRouteListResponse200]:
    """List Docs owned by a User

    Args:
        id (str):
        limit (int): Limit the number of results
        offset (int): Offset to apply to the results
        sort_by (UserDocsRouteListSortBy):  Default: UserDocsRouteListSortBy.CREATED_AT.
        direction (UserDocsRouteListDirection):  Default: UserDocsRouteListDirection.ASC.
        metadata_filter (str):  Default: '{}'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        UserDocsRouteListResponse200
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            limit=limit,
            offset=offset,
            sort_by=sort_by,
            direction=direction,
            metadata_filter=metadata_filter,
        )
    ).parsed



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.context_overflow_type import ContextOverflowType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.patch_session_request_metadata import PatchSessionRequestMetadata


T = TypeVar("T", bound="PatchSessionRequest")


@_attrs_define
class PatchSessionRequest:
    r"""Payload for patching a session

    Attributes:
        situation (Union[Unset, str]): A specific situation that sets the background for this session Default: '{%- if
            agent.name -%}\nYou are {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if agent.about -%}\nAbout you:
            {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if user -%}\nYou are talking to a user\n  {%- if user.name
            -%}{{\\" \\"}} and their name is {{user.name}}\n    {%- if user.about -%}. About the user: {{user.about}}.{%-
            else -%}.{%- endif -%}\n  {%- endif -%}\n{%- endif -%}\n\n{{\\"\n\n\\"}}\n\n{%- if agent.instructions
            -%}\nInstructions:{{\\"\n\\"}}\n  {%- if agent.instructions is string -%}\n
            {{agent.instructions}}{{\\"\n\\"}}\n  {%- else -%}\n    {%- for instruction in agent.instructions -%}\n      -
            {{instruction}}{{\\"\n\\"}}\n    {%- endfor -%}\n  {%- endif -%}\n  {{\\"\n\\"}}\n{%- endif -%}\n\n{%- if tools
            -%}\nTools:{{\\"\n\\"}}\n  {%- for tool in tools -%}\n    {%- if tool.type == \\"function\\" -%}\n      -
            {{tool.function.name}}\n      {%- if tool.function.description -%}: {{tool.function.description}}{%- endif
            -%}{{\\"\n\\"}}\n    {%- else -%}\n      - {{ 0/0 }} {# Error: Other tool types aren\'t supported yet. #}\n
            {%- endif -%}\n  {%- endfor -%}\n{{\\"\n\n\\"}}\n{%- endif -%}\n\n{%- if docs -%}\nRelevant
            documents:{{\\"\n\\"}}\n  {%- for doc in docs -%}\n    {{doc.title}}{{\\"\n\\"}}\n    {%- if doc.content is
            string -%}\n      {{doc.content}}{{\\"\n\\"}}\n    {%- else -%}\n      {%- for snippet in doc.content -%}\n
            {{snippet}}{{\\"\n\\"}}\n      {%- endfor -%}\n    {%- endif -%}\n    {{\\"---\\"}}\n  {%- endfor -%}\n{%- endif
            -%}'.
        render_templates (Union[Unset, bool]): Render system and assistant message content as jinja templates Default:
            True.
        token_budget (Union[None, Unset, int]): Threshold value for the adaptive context functionality
        context_overflow (Union[ContextOverflowType, None, Unset]): Action to start on context window overflow
        metadata (Union[Unset, PatchSessionRequestMetadata]):
    """

    situation: Union[Unset, str] = (
        '{%- if agent.name -%}\nYou are {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if agent.about -%}\nAbout you: {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if user -%}\nYou are talking to a user\n  {%- if user.name -%}{{\\" \\"}} and their name is {{user.name}}\n    {%- if user.about -%}. About the user: {{user.about}}.{%- else -%}.{%- endif -%}\n  {%- endif -%}\n{%- endif -%}\n\n{{\\"\n\n\\"}}\n\n{%- if agent.instructions -%}\nInstructions:{{\\"\n\\"}}\n  {%- if agent.instructions is string -%}\n    {{agent.instructions}}{{\\"\n\\"}}\n  {%- else -%}\n    {%- for instruction in agent.instructions -%}\n      - {{instruction}}{{\\"\n\\"}}\n    {%- endfor -%}\n  {%- endif -%}\n  {{\\"\n\\"}}\n{%- endif -%}\n\n{%- if tools -%}\nTools:{{\\"\n\\"}}\n  {%- for tool in tools -%}\n    {%- if tool.type == \\"function\\" -%}\n      - {{tool.function.name}}\n      {%- if tool.function.description -%}: {{tool.function.description}}{%- endif -%}{{\\"\n\\"}}\n    {%- else -%}\n      - {{ 0/0 }} {# Error: Other tool types aren\'t supported yet. #}\n    {%- endif -%}\n  {%- endfor -%}\n{{\\"\n\n\\"}}\n{%- endif -%}\n\n{%- if docs -%}\nRelevant documents:{{\\"\n\\"}}\n  {%- for doc in docs -%}\n    {{doc.title}}{{\\"\n\\"}}\n    {%- if doc.content is string -%}\n      {{doc.content}}{{\\"\n\\"}}\n    {%- else -%}\n      {%- for snippet in doc.content -%}\n        {{snippet}}{{\\"\n\\"}}\n      {%- endfor -%}\n    {%- endif -%}\n    {{\\"---\\"}}\n  {%- endfor -%}\n{%- endif -%}'
    )
    render_templates: Union[Unset, bool] = True
    token_budget: Union[None, Unset, int] = UNSET
    context_overflow: Union[ContextOverflowType, None, Unset] = UNSET
    metadata: Union[Unset, "PatchSessionRequestMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.patch_session_request_metadata import PatchSessionRequestMetadata

        situation = self.situation

        render_templates = self.render_templates

        token_budget: Union[None, Unset, int]
        if isinstance(self.token_budget, Unset):
            token_budget = UNSET
        else:
            token_budget = self.token_budget

        context_overflow: Union[None, Unset, str]
        if isinstance(self.context_overflow, Unset):
            context_overflow = UNSET
        elif isinstance(self.context_overflow, ContextOverflowType):
            context_overflow = self.context_overflow.value
        else:
            context_overflow = self.context_overflow

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if situation is not UNSET:
            field_dict["situation"] = situation
        if render_templates is not UNSET:
            field_dict["render_templates"] = render_templates
        if token_budget is not UNSET:
            field_dict["token_budget"] = token_budget
        if context_overflow is not UNSET:
            field_dict["context_overflow"] = context_overflow
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.patch_session_request_metadata import PatchSessionRequestMetadata

        d = src_dict.copy()
        situation = d.pop("situation", UNSET)

        render_templates = d.pop("render_templates", UNSET)

        def _parse_token_budget(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        token_budget = _parse_token_budget(d.pop("token_budget", UNSET))

        def _parse_context_overflow(
            data: object,
        ) -> Union[ContextOverflowType, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                context_overflow_type_0 = ContextOverflowType(data)

                return context_overflow_type_0
            except:  # noqa: E722
                pass
            return cast(Union[ContextOverflowType, None, Unset], data)

        context_overflow = _parse_context_overflow(d.pop("context_overflow", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, PatchSessionRequestMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = PatchSessionRequestMetadata.from_dict(_metadata)

        patch_session_request = cls(
            situation=situation,
            render_templates=render_templates,
            token_budget=token_budget,
            context_overflow=context_overflow,
            metadata=metadata,
        )

        patch_session_request.additional_properties = d
        return patch_session_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.prompt_step_kind import PromptStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_settings import ChatSettings
    from ..models.prompt_step_prompt_type_1_item import PromptStepPromptType1Item


T = TypeVar("T", bound="PromptStep")


@_attrs_define
class PromptStep:
    """
    Attributes:
        kind (PromptStepKind): The kind of step Default: PromptStepKind.PROMPT.
        prompt (Union[List['PromptStepPromptType1Item'], str]): The prompt to run
        settings (Union[Unset, ChatSettings]):
    """

    prompt: Union[List["PromptStepPromptType1Item"], str]
    kind: PromptStepKind = PromptStepKind.PROMPT
    settings: Union[Unset, "ChatSettings"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_settings import ChatSettings
        from ..models.prompt_step_prompt_type_1_item import PromptStepPromptType1Item

        kind = self.kind.value

        prompt: Union[List[Dict[str, Any]], str]
        if isinstance(self.prompt, list):
            prompt = []
            for prompt_type_1_item_data in self.prompt:
                prompt_type_1_item = prompt_type_1_item_data.to_dict()
                prompt.append(prompt_type_1_item)

        else:
            prompt = self.prompt

        settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "prompt": prompt,
            }
        )
        if settings is not UNSET:
            field_dict["settings"] = settings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_settings import ChatSettings
        from ..models.prompt_step_prompt_type_1_item import PromptStepPromptType1Item

        d = src_dict.copy()
        kind = PromptStepKind(d.pop("kind_"))

        def _parse_prompt(
            data: object,
        ) -> Union[List["PromptStepPromptType1Item"], str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                prompt_type_1 = []
                _prompt_type_1 = data
                for prompt_type_1_item_data in _prompt_type_1:
                    prompt_type_1_item = PromptStepPromptType1Item.from_dict(
                        prompt_type_1_item_data
                    )

                    prompt_type_1.append(prompt_type_1_item)

                return prompt_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List["PromptStepPromptType1Item"], str], data)

        prompt = _parse_prompt(d.pop("prompt"))

        _settings = d.pop("settings", UNSET)
        settings: Union[Unset, ChatSettings]
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = ChatSettings.from_dict(_settings)

        prompt_step = cls(
            kind=kind,
            prompt=prompt,
            settings=settings,
        )

        prompt_step.additional_properties = d
        return prompt_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.tool_call_step_arguments_type_1 import ToolCallStepArgumentsType1
from ..models.tool_call_step_kind import ToolCallStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tool_call_step_arguments_type_0 import ToolCallStepArgumentsType0


T = TypeVar("T", bound="ToolCallStep")


@_attrs_define
class ToolCallStep:
    """
    Attributes:
        kind (ToolCallStepKind): The kind of step Default: ToolCallStepKind.TOOL_CALL.
        tool (str): Naming convention for tool references. Tools are resolved in order: `step-settings` -> `task` ->
            `agent`
        arguments (Union['ToolCallStepArgumentsType0', ToolCallStepArgumentsType1]): The input parameters for the tool
            (defaults to last step output) Default: ToolCallStepArgumentsType1.VALUE_0.
    """

    tool: str
    kind: ToolCallStepKind = ToolCallStepKind.TOOL_CALL
    arguments: Union["ToolCallStepArgumentsType0", ToolCallStepArgumentsType1] = (
        ToolCallStepArgumentsType1.VALUE_0
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.tool_call_step_arguments_type_0 import ToolCallStepArgumentsType0

        kind = self.kind.value

        tool = self.tool

        arguments: Union[Dict[str, Any], str]
        if isinstance(self.arguments, ToolCallStepArgumentsType0):
            arguments = self.arguments.to_dict()
        else:
            arguments = self.arguments.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "tool": tool,
                "arguments": arguments,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tool_call_step_arguments_type_0 import ToolCallStepArgumentsType0

        d = src_dict.copy()
        kind = ToolCallStepKind(d.pop("kind_"))

        tool = d.pop("tool")

        def _parse_arguments(
            data: object,
        ) -> Union["ToolCallStepArgumentsType0", ToolCallStepArgumentsType1]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                arguments_type_0 = ToolCallStepArgumentsType0.from_dict(data)

                return arguments_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, str):
                raise TypeError()
            arguments_type_1 = ToolCallStepArgumentsType1(data)

            return arguments_type_1

        arguments = _parse_arguments(d.pop("arguments"))

        tool_call_step = cls(
            kind=kind,
            tool=tool,
            arguments=arguments,
        )

        tool_call_step.additional_properties = d
        return tool_call_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.if_else_workflow_step_kind import IfElseWorkflowStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embed_step import EmbedStep
    from ..models.error_workflow_step import ErrorWorkflowStep
    from ..models.evaluate_step import EvaluateStep
    from ..models.get_step import GetStep
    from ..models.log_step import LogStep
    from ..models.prompt_step import PromptStep
    from ..models.return_step import ReturnStep
    from ..models.search_step import SearchStep
    from ..models.set_step import SetStep
    from ..models.sleep_step import SleepStep
    from ..models.tool_call_step import ToolCallStep
    from ..models.wait_for_input_step import WaitForInputStep
    from ..models.yield_step import YieldStep


T = TypeVar("T", bound="IfElseWorkflowStep")


@_attrs_define
class IfElseWorkflowStep:
    """
    Attributes:
        kind (IfElseWorkflowStepKind): The kind of step Default: IfElseWorkflowStepKind.IF_ELSE.
        if_ (str): A simple python expression compatible with SimpleEval.
        then (Union['EmbedStep', 'ErrorWorkflowStep', 'EvaluateStep', 'GetStep', 'LogStep', 'PromptStep', 'ReturnStep',
            'SearchStep', 'SetStep', 'SleepStep', 'ToolCallStep', 'WaitForInputStep', 'YieldStep']): The steps to run if the
            condition is true
        else_ (Union['EmbedStep', 'ErrorWorkflowStep', 'EvaluateStep', 'GetStep', 'LogStep', 'PromptStep', 'ReturnStep',
            'SearchStep', 'SetStep', 'SleepStep', 'ToolCallStep', 'WaitForInputStep', 'YieldStep', None]): The steps to run
            if the condition is false
    """

    if_: str
    then: Union[
        "EmbedStep",
        "ErrorWorkflowStep",
        "EvaluateStep",
        "GetStep",
        "LogStep",
        "PromptStep",
        "ReturnStep",
        "SearchStep",
        "SetStep",
        "SleepStep",
        "ToolCallStep",
        "WaitForInputStep",
        "YieldStep",
    ]
    else_: Union[
        "EmbedStep",
        "ErrorWorkflowStep",
        "EvaluateStep",
        "GetStep",
        "LogStep",
        "PromptStep",
        "ReturnStep",
        "SearchStep",
        "SetStep",
        "SleepStep",
        "ToolCallStep",
        "WaitForInputStep",
        "YieldStep",
        None,
    ]
    kind: IfElseWorkflowStepKind = IfElseWorkflowStepKind.IF_ELSE
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.embed_step import EmbedStep
        from ..models.error_workflow_step import ErrorWorkflowStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.return_step import ReturnStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.sleep_step import SleepStep
        from ..models.tool_call_step import ToolCallStep
        from ..models.wait_for_input_step import WaitForInputStep
        from ..models.yield_step import YieldStep

        kind = self.kind.value

        if_ = self.if_

        then: Dict[str, Any]
        if isinstance(self.then, EvaluateStep):
            then = self.then.to_dict()
        elif isinstance(self.then, ToolCallStep):
            then = self.then.to_dict()
        elif isinstance(self.then, PromptStep):
            then = self.then.to_dict()
        elif isinstance(self.then, GetStep):
            then = self.then.to_dict()
        elif isinstance(self.then, SetStep):
            then = self.then.to_dict()
        elif isinstance(self.then, LogStep):
            then = self.then.to_dict()
        elif isinstance(self.then, EmbedStep):
            then = self.then.to_dict()
        elif isinstance(self.then, SearchStep):
            then = self.then.to_dict()
        elif isinstance(self.then, ReturnStep):
            then = self.then.to_dict()
        elif isinstance(self.then, SleepStep):
            then = self.then.to_dict()
        elif isinstance(self.then, ErrorWorkflowStep):
            then = self.then.to_dict()
        elif isinstance(self.then, YieldStep):
            then = self.then.to_dict()
        else:
            then = self.then.to_dict()

        else_: Union[Dict[str, Any], None]
        if isinstance(self.else_, EvaluateStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, ToolCallStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, PromptStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, GetStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, SetStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, LogStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, EmbedStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, SearchStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, ReturnStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, SleepStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, ErrorWorkflowStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, YieldStep):
            else_ = self.else_.to_dict()
        elif isinstance(self.else_, WaitForInputStep):
            else_ = self.else_.to_dict()
        else:
            else_ = self.else_

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "if": if_,
                "then": then,
                "else": else_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embed_step import EmbedStep
        from ..models.error_workflow_step import ErrorWorkflowStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.return_step import ReturnStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.sleep_step import SleepStep
        from ..models.tool_call_step import ToolCallStep
        from ..models.wait_for_input_step import WaitForInputStep
        from ..models.yield_step import YieldStep

        d = src_dict.copy()
        kind = IfElseWorkflowStepKind(d.pop("kind_"))

        if_ = d.pop("if")

        def _parse_then(
            data: object,
        ) -> Union[
            "EmbedStep",
            "ErrorWorkflowStep",
            "EvaluateStep",
            "GetStep",
            "LogStep",
            "PromptStep",
            "ReturnStep",
            "SearchStep",
            "SetStep",
            "SleepStep",
            "ToolCallStep",
            "WaitForInputStep",
            "YieldStep",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_0 = EvaluateStep.from_dict(data)

                return then_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_1 = ToolCallStep.from_dict(data)

                return then_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_2 = PromptStep.from_dict(data)

                return then_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_3 = GetStep.from_dict(data)

                return then_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_4 = SetStep.from_dict(data)

                return then_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_5 = LogStep.from_dict(data)

                return then_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_6 = EmbedStep.from_dict(data)

                return then_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_7 = SearchStep.from_dict(data)

                return then_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_8 = ReturnStep.from_dict(data)

                return then_type_8
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_9 = SleepStep.from_dict(data)

                return then_type_9
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_10 = ErrorWorkflowStep.from_dict(data)

                return then_type_10
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                then_type_11 = YieldStep.from_dict(data)

                return then_type_11
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            then_type_12 = WaitForInputStep.from_dict(data)

            return then_type_12

        then = _parse_then(d.pop("then"))

        def _parse_else_(
            data: object,
        ) -> Union[
            "EmbedStep",
            "ErrorWorkflowStep",
            "EvaluateStep",
            "GetStep",
            "LogStep",
            "PromptStep",
            "ReturnStep",
            "SearchStep",
            "SetStep",
            "SleepStep",
            "ToolCallStep",
            "WaitForInputStep",
            "YieldStep",
            None,
        ]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_0 = EvaluateStep.from_dict(data)

                return else_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_1 = ToolCallStep.from_dict(data)

                return else_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_2 = PromptStep.from_dict(data)

                return else_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_3 = GetStep.from_dict(data)

                return else_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_4 = SetStep.from_dict(data)

                return else_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_5 = LogStep.from_dict(data)

                return else_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_6 = EmbedStep.from_dict(data)

                return else_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_7 = SearchStep.from_dict(data)

                return else_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_8 = ReturnStep.from_dict(data)

                return else_type_8
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_9 = SleepStep.from_dict(data)

                return else_type_9
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_10 = ErrorWorkflowStep.from_dict(data)

                return else_type_10
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_11 = YieldStep.from_dict(data)

                return else_type_11
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                else_type_12 = WaitForInputStep.from_dict(data)

                return else_type_12
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "EmbedStep",
                    "ErrorWorkflowStep",
                    "EvaluateStep",
                    "GetStep",
                    "LogStep",
                    "PromptStep",
                    "ReturnStep",
                    "SearchStep",
                    "SetStep",
                    "SleepStep",
                    "ToolCallStep",
                    "WaitForInputStep",
                    "YieldStep",
                    None,
                ],
                data,
            )

        else_ = _parse_else_(d.pop("else"))

        if_else_workflow_step = cls(
            kind=kind,
            if_=if_,
            then=then,
            else_=else_,
        )

        if_else_workflow_step.additional_properties = d
        return if_else_workflow_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.task_additional_property_item_type_17_kind import (
    TaskAdditionalPropertyItemType17Kind,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embed_step import EmbedStep
    from ..models.evaluate_step import EvaluateStep
    from ..models.get_step import GetStep
    from ..models.log_step import LogStep
    from ..models.prompt_step import PromptStep
    from ..models.search_step import SearchStep
    from ..models.set_step import SetStep
    from ..models.tool_call_step import ToolCallStep


T = TypeVar("T", bound="TaskAdditionalPropertyItemType17")


@_attrs_define
class TaskAdditionalPropertyItemType17:
    """
    Attributes:
        kind (TaskAdditionalPropertyItemType17Kind): The kind of step Default:
            TaskAdditionalPropertyItemType17Kind.MAP_REDUCE.
        over (str): A simple python expression compatible with SimpleEval.
        map_ (Union['EmbedStep', 'EvaluateStep', 'GetStep', 'LogStep', 'PromptStep', 'SearchStep', 'SetStep',
            'ToolCallStep']): The steps to run for each iteration
        reduce (Union[Unset, str]): A simple python expression compatible with SimpleEval.
        initial (Union[Unset, Any]): The initial value of the reduce expression Default: [].
        parallelism (Union[Unset, int]): Whether to run the reduce expression in parallel and how many items to run in
            each batch
    """

    over: str
    map_: Union[
        "EmbedStep",
        "EvaluateStep",
        "GetStep",
        "LogStep",
        "PromptStep",
        "SearchStep",
        "SetStep",
        "ToolCallStep",
    ]
    kind: TaskAdditionalPropertyItemType17Kind = (
        TaskAdditionalPropertyItemType17Kind.MAP_REDUCE
    )
    reduce: Union[Unset, str] = UNSET
    initial: Union[Unset, Any] = []
    parallelism: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.embed_step import EmbedStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.tool_call_step import ToolCallStep

        kind = self.kind.value

        over = self.over

        map_: Dict[str, Any]
        if isinstance(self.map_, EvaluateStep):
            map_ = self.map_.to_dict()
        elif isinstance(self.map_, ToolCallStep):
            map_ = self.map_.to_dict()
        elif isinstance(self.map_, PromptStep):
            map_ = self.map_.to_dict()
        elif isinstance(self.map_, GetStep):
            map_ = self.map_.to_dict()
        elif isinstance(self.map_, SetStep):
            map_ = self.map_.to_dict()
        elif isinstance(self.map_, LogStep):
            map_ = self.map_.to_dict()
        elif isinstance(self.map_, EmbedStep):
            map_ = self.map_.to_dict()
        else:
            map_ = self.map_.to_dict()

        reduce = self.reduce

        initial = self.initial

        parallelism = self.parallelism

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "over": over,
                "map": map_,
            }
        )
        if reduce is not UNSET:
            field_dict["reduce"] = reduce
        if initial is not UNSET:
            field_dict["initial"] = initial
        if parallelism is not UNSET:
            field_dict["parallelism"] = parallelism

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embed_step import EmbedStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.tool_call_step import ToolCallStep

        d = src_dict.copy()
        kind = TaskAdditionalPropertyItemType17Kind(d.pop("kind_"))

        over = d.pop("over")

        def _parse_map_(
            data: object,
        ) -> Union[
            "EmbedStep",
            "EvaluateStep",
            "GetStep",
            "LogStep",
            "PromptStep",
            "SearchStep",
            "SetStep",
            "ToolCallStep",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_0 = EvaluateStep.from_dict(data)

                return map_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_1 = ToolCallStep.from_dict(data)

                return map_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_2 = PromptStep.from_dict(data)

                return map_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_3 = GetStep.from_dict(data)

                return map_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_4 = SetStep.from_dict(data)

                return map_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_5 = LogStep.from_dict(data)

                return map_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                map_type_6 = EmbedStep.from_dict(data)

                return map_type_6
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            map_type_7 = SearchStep.from_dict(data)

            return map_type_7

        map_ = _parse_map_(d.pop("map"))

        reduce = d.pop("reduce", UNSET)

        initial = d.pop("initial", UNSET)

        parallelism = d.pop("parallelism", UNSET)

        task_additional_property_item_type_17 = cls(
            kind=kind,
            over=over,
            map_=map_,
            reduce=reduce,
            initial=initial,
            parallelism=parallelism,
        )

        task_additional_property_item_type_17.additional_properties = d
        return task_additional_property_item_type_17

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.update_user_request_metadata import UpdateUserRequestMetadata


T = TypeVar("T", bound="UpdateUserRequest")


@_attrs_define
class UpdateUserRequest:
    """Payload for updating a user

    Attributes:
        name (str): For Unicode character safety
            See: https://unicode.org/reports/tr31/
            See: https://www.unicode.org/reports/tr39/#Identifier_Characters Default: ''.
        about (str): About the user Default: ''.
        metadata (Union[Unset, UpdateUserRequestMetadata]):
    """

    name: str = ""
    about: str = ""
    metadata: Union[Unset, "UpdateUserRequestMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.update_user_request_metadata import UpdateUserRequestMetadata

        name = self.name

        about = self.about

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "about": about,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.update_user_request_metadata import UpdateUserRequestMetadata

        d = src_dict.copy()
        name = d.pop("name")

        about = d.pop("about")

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, UpdateUserRequestMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = UpdateUserRequestMetadata.from_dict(_metadata)

        update_user_request = cls(
            name=name,
            about=about,
            metadata=metadata,
        )

        update_user_request.additional_properties = d
        return update_user_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.parallel_step_kind import ParallelStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embed_step import EmbedStep
    from ..models.evaluate_step import EvaluateStep
    from ..models.get_step import GetStep
    from ..models.log_step import LogStep
    from ..models.prompt_step import PromptStep
    from ..models.search_step import SearchStep
    from ..models.set_step import SetStep
    from ..models.tool_call_step import ToolCallStep


T = TypeVar("T", bound="ParallelStep")


@_attrs_define
class ParallelStep:
    """
    Attributes:
        kind (ParallelStepKind): The kind of step Default: ParallelStepKind.PARALLEL.
        parallel (List[Union['EmbedStep', 'EvaluateStep', 'GetStep', 'LogStep', 'PromptStep', 'SearchStep', 'SetStep',
            'ToolCallStep']]): The steps to run in parallel. Max concurrency will depend on the platform.
    """

    parallel: List[
        Union[
            "EmbedStep",
            "EvaluateStep",
            "GetStep",
            "LogStep",
            "PromptStep",
            "SearchStep",
            "SetStep",
            "ToolCallStep",
        ]
    ]
    kind: ParallelStepKind = ParallelStepKind.PARALLEL
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.embed_step import EmbedStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.tool_call_step import ToolCallStep

        kind = self.kind.value

        parallel = []
        for parallel_item_data in self.parallel:
            parallel_item: Dict[str, Any]
            if isinstance(parallel_item_data, EvaluateStep):
                parallel_item = parallel_item_data.to_dict()
            elif isinstance(parallel_item_data, ToolCallStep):
                parallel_item = parallel_item_data.to_dict()
            elif isinstance(parallel_item_data, PromptStep):
                parallel_item = parallel_item_data.to_dict()
            elif isinstance(parallel_item_data, GetStep):
                parallel_item = parallel_item_data.to_dict()
            elif isinstance(parallel_item_data, SetStep):
                parallel_item = parallel_item_data.to_dict()
            elif isinstance(parallel_item_data, LogStep):
                parallel_item = parallel_item_data.to_dict()
            elif isinstance(parallel_item_data, EmbedStep):
                parallel_item = parallel_item_data.to_dict()
            else:
                parallel_item = parallel_item_data.to_dict()

            parallel.append(parallel_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "parallel": parallel,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embed_step import EmbedStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.tool_call_step import ToolCallStep

        d = src_dict.copy()
        kind = ParallelStepKind(d.pop("kind_"))

        parallel = []
        _parallel = d.pop("parallel")
        for parallel_item_data in _parallel:

            def _parse_parallel_item(
                data: object,
            ) -> Union[
                "EmbedStep",
                "EvaluateStep",
                "GetStep",
                "LogStep",
                "PromptStep",
                "SearchStep",
                "SetStep",
                "ToolCallStep",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_0 = EvaluateStep.from_dict(data)

                    return parallel_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_1 = ToolCallStep.from_dict(data)

                    return parallel_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_2 = PromptStep.from_dict(data)

                    return parallel_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_3 = GetStep.from_dict(data)

                    return parallel_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_4 = SetStep.from_dict(data)

                    return parallel_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_5 = LogStep.from_dict(data)

                    return parallel_item_type_5
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    parallel_item_type_6 = EmbedStep.from_dict(data)

                    return parallel_item_type_6
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                parallel_item_type_7 = SearchStep.from_dict(data)

                return parallel_item_type_7

            parallel_item = _parse_parallel_item(parallel_item_data)

            parallel.append(parallel_item)

        parallel_step = cls(
            kind=kind,
            parallel=parallel,
        )

        parallel_step.additional_properties = d
        return parallel_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.job_state import JobState
from ..types import UNSET, Unset

T = TypeVar("T", bound="JobStatus")


@_attrs_define
class JobStatus:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime): When this resource was created as UTC date-time
        updated_at (datetime.datetime): When this resource was updated as UTC date-time
        name (str): For Unicode character safety
            See: https://unicode.org/reports/tr31/
            See: https://www.unicode.org/reports/tr39/#Identifier_Characters Default: ''.
        reason (str): Reason for the current state of the job Default: ''.
        has_progress (bool): Whether this Job supports progress updates Default: False.
        progress (float): Progress percentage Default: 0.0.
        state (JobState): Current state (one of: pending, in_progress, retrying, succeeded, aborted, failed) Default:
            JobState.PENDING.
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    name: str = ""
    reason: str = ""
    has_progress: bool = False
    progress: float = 0.0
    state: JobState = JobState.PENDING
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        name = self.name

        reason = self.reason

        has_progress = self.has_progress

        progress = self.progress

        state = self.state.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "name": name,
                "reason": reason,
                "has_progress": has_progress,
                "progress": progress,
                "state": state,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        name = d.pop("name")

        reason = d.pop("reason")

        has_progress = d.pop("has_progress")

        progress = d.pop("progress")

        state = JobState(d.pop("state"))

        job_status = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            name=name,
            reason=reason,
            has_progress=has_progress,
            progress=progress,
            state=state,
        )

        job_status.additional_properties = d
        return job_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.doc_metadata import DocMetadata


T = TypeVar("T", bound="Doc")


@_attrs_define
class Doc:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime): When this resource was created as UTC date-time
        title (str): Title describing what this document contains
        content (Union[List[str], str]): Contents of the document
        metadata (Union[Unset, DocMetadata]):
    """

    id: str
    created_at: datetime.datetime
    title: str
    content: Union[List[str], str]
    metadata: Union[Unset, "DocMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.doc_metadata import DocMetadata

        id = self.id

        created_at = self.created_at.isoformat()

        title = self.title

        content: Union[List[str], str]
        if isinstance(self.content, list):
            content = self.content

        else:
            content = self.content

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "title": title,
                "content": content,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.doc_metadata import DocMetadata

        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        title = d.pop("title")

        def _parse_content(data: object) -> Union[List[str], str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_1 = cast(List[str], data)

                return content_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], str], data)

        content = _parse_content(d.pop("content"))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, DocMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = DocMetadata.from_dict(_metadata)

        doc = cls(
            id=id,
            created_at=created_at,
            title=title,
            content=content,
            metadata=metadata,
        )

        doc.additional_properties = d
        return doc

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class JobState(str, Enum):
    ABORTED = "aborted"
    FAILED = "failed"
    IN_PROGRESS = "in_progress"
    PENDING = "pending"
    RETRYING = "retrying"
    SUCCEEDED = "succeeded"
    UNKNOWN = "unknown"

    def __str__(self) -> str:
        return str(self.value)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_execution_request_input import CreateExecutionRequestInput
    from ..models.create_execution_request_metadata import (
        CreateExecutionRequestMetadata,
    )


T = TypeVar("T", bound="CreateExecutionRequest")


@_attrs_define
class CreateExecutionRequest:
    """Payload for creating an execution

    Attributes:
        input_ (CreateExecutionRequestInput): The input to the execution
        output (Union[Unset, Any]): The output of the execution if it succeeded
        error (Union[Unset, str]): The error of the execution if it failed
        metadata (Union[Unset, CreateExecutionRequestMetadata]):
    """

    input_: "CreateExecutionRequestInput"
    output: Union[Unset, Any] = UNSET
    error: Union[Unset, str] = UNSET
    metadata: Union[Unset, "CreateExecutionRequestMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.create_execution_request_input import CreateExecutionRequestInput
        from ..models.create_execution_request_metadata import (
            CreateExecutionRequestMetadata,
        )

        input_ = self.input_.to_dict()

        output = self.output

        error = self.error

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "input": input_,
            }
        )
        if output is not UNSET:
            field_dict["output"] = output
        if error is not UNSET:
            field_dict["error"] = error
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_execution_request_input import CreateExecutionRequestInput
        from ..models.create_execution_request_metadata import (
            CreateExecutionRequestMetadata,
        )

        d = src_dict.copy()
        input_ = CreateExecutionRequestInput.from_dict(d.pop("input"))

        output = d.pop("output", UNSET)

        error = d.pop("error", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, CreateExecutionRequestMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = CreateExecutionRequestMetadata.from_dict(_metadata)

        create_execution_request = cls(
            input_=input_,
            output=output,
            error=error,
            metadata=metadata,
        )

        create_execution_request.additional_properties = d
        return create_execution_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.doc_owner import DocOwner
    from ..models.snippet import Snippet


T = TypeVar("T", bound="DocReference")


@_attrs_define
class DocReference:
    """
    Attributes:
        owner (DocOwner):
        id (str):
        snippets (List['Snippet']):
        distance (Union[None, float]):
        title (Union[Unset, str]):
    """

    owner: "DocOwner"
    id: str
    snippets: List["Snippet"]
    distance: Union[None, float]
    title: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.doc_owner import DocOwner
        from ..models.snippet import Snippet

        owner = self.owner.to_dict()

        id = self.id

        snippets = []
        for snippets_item_data in self.snippets:
            snippets_item = snippets_item_data.to_dict()
            snippets.append(snippets_item)

        distance: Union[None, float]
        distance = self.distance

        title = self.title

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "owner": owner,
                "id": id,
                "snippets": snippets,
                "distance": distance,
            }
        )
        if title is not UNSET:
            field_dict["title"] = title

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.doc_owner import DocOwner
        from ..models.snippet import Snippet

        d = src_dict.copy()
        owner = DocOwner.from_dict(d.pop("owner"))

        id = d.pop("id")

        snippets = []
        _snippets = d.pop("snippets")
        for snippets_item_data in _snippets:
            snippets_item = Snippet.from_dict(snippets_item_data)

            snippets.append(snippets_item)

        def _parse_distance(data: object) -> Union[None, float]:
            if data is None:
                return data
            return cast(Union[None, float], data)

        distance = _parse_distance(d.pop("distance"))

        title = d.pop("title", UNSET)

        doc_reference = cls(
            owner=owner,
            id=id,
            snippets=snippets,
            distance=distance,
            title=title,
        )

        doc_reference.additional_properties = d
        return doc_reference

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EntriesChatMLRole(str, Enum):
    ASSISTANT = "assistant"
    AUTO = "auto"
    FUNCTION = "function"
    FUNCTION_CALL = "function_call"
    FUNCTION_RESPONSE = "function_response"
    SYSTEM = "system"
    USER = "user"

    def __str__(self) -> str:
        return str(self.value)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmbedQueryRequest")


@_attrs_define
class EmbedQueryRequest:
    """
    Attributes:
        text (Union[List[str], str]): Text or texts to embed
    """

    text: Union[List[str], str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        text: Union[List[str], str]
        if isinstance(self.text, list):
            text = self.text

        else:
            text = self.text

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "text": text,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_text(data: object) -> Union[List[str], str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                text_type_1 = cast(List[str], data)

                return text_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], str], data)

        text = _parse_text(d.pop("text"))

        embed_query_request = cls(
            text=text,
        )

        embed_query_request.additional_properties = d
        return embed_query_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.prompt_step_prompt_type_1_item_content_type_2_item_type_0_type import (
    PromptStepPromptType1ItemContentType2ItemType0Type,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="PromptStepPromptType1ItemContentType2ItemType0")


@_attrs_define
class PromptStepPromptType1ItemContentType2ItemType0:
    """
    Attributes:
        text (str): A valid jinja template.
        type (PromptStepPromptType1ItemContentType2ItemType0Type): The type (fixed to 'text') Default:
            PromptStepPromptType1ItemContentType2ItemType0Type.TEXT.
    """

    text: str
    type: PromptStepPromptType1ItemContentType2ItemType0Type = (
        PromptStepPromptType1ItemContentType2ItemType0Type.TEXT
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        text = self.text

        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "text": text,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        text = d.pop("text")

        type = PromptStepPromptType1ItemContentType2ItemType0Type(d.pop("type"))

        prompt_step_prompt_type_1_item_content_type_2_item_type_0 = cls(
            text=text,
            type=type,
        )

        prompt_step_prompt_type_1_item_content_type_2_item_type_0.additional_properties = d
        return prompt_step_prompt_type_1_item_content_type_2_item_type_0

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embed_step import EmbedStep
    from ..models.evaluate_step import EvaluateStep
    from ..models.get_step import GetStep
    from ..models.log_step import LogStep
    from ..models.prompt_step import PromptStep
    from ..models.search_step import SearchStep
    from ..models.set_step import SetStep
    from ..models.tool_call_step import ToolCallStep


T = TypeVar("T", bound="ForeachDo")


@_attrs_define
class ForeachDo:
    """
    Attributes:
        in_ (str): A simple python expression compatible with SimpleEval.
        do (Union['EmbedStep', 'EvaluateStep', 'GetStep', 'LogStep', 'PromptStep', 'SearchStep', 'SetStep',
            'ToolCallStep']): The steps to run for each iteration
    """

    in_: str
    do: Union[
        "EmbedStep",
        "EvaluateStep",
        "GetStep",
        "LogStep",
        "PromptStep",
        "SearchStep",
        "SetStep",
        "ToolCallStep",
    ]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.embed_step import EmbedStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.tool_call_step import ToolCallStep

        in_ = self.in_

        do: Dict[str, Any]
        if isinstance(self.do, EvaluateStep):
            do = self.do.to_dict()
        elif isinstance(self.do, ToolCallStep):
            do = self.do.to_dict()
        elif isinstance(self.do, PromptStep):
            do = self.do.to_dict()
        elif isinstance(self.do, GetStep):
            do = self.do.to_dict()
        elif isinstance(self.do, SetStep):
            do = self.do.to_dict()
        elif isinstance(self.do, LogStep):
            do = self.do.to_dict()
        elif isinstance(self.do, EmbedStep):
            do = self.do.to_dict()
        else:
            do = self.do.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "in": in_,
                "do": do,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embed_step import EmbedStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.get_step import GetStep
        from ..models.log_step import LogStep
        from ..models.prompt_step import PromptStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.tool_call_step import ToolCallStep

        d = src_dict.copy()
        in_ = d.pop("in")

        def _parse_do(
            data: object,
        ) -> Union[
            "EmbedStep",
            "EvaluateStep",
            "GetStep",
            "LogStep",
            "PromptStep",
            "SearchStep",
            "SetStep",
            "ToolCallStep",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_0 = EvaluateStep.from_dict(data)

                return do_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_1 = ToolCallStep.from_dict(data)

                return do_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_2 = PromptStep.from_dict(data)

                return do_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_3 = GetStep.from_dict(data)

                return do_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_4 = SetStep.from_dict(data)

                return do_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_5 = LogStep.from_dict(data)

                return do_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                do_type_6 = EmbedStep.from_dict(data)

                return do_type_6
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            do_type_7 = SearchStep.from_dict(data)

            return do_type_7

        do = _parse_do(d.pop("do"))

        foreach_do = cls(
            in_=in_,
            do=do,
        )

        foreach_do.additional_properties = d
        return foreach_do

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.schema_completion_response_format_type import (
    SchemaCompletionResponseFormatType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.schema_completion_response_format_json_schema import (
        SchemaCompletionResponseFormatJsonSchema,
    )


T = TypeVar("T", bound="SchemaCompletionResponseFormat")


@_attrs_define
class SchemaCompletionResponseFormat:
    """
    Attributes:
        type (SchemaCompletionResponseFormatType): The format of the response Default:
            SchemaCompletionResponseFormatType.JSON_SCHEMA.
        json_schema (SchemaCompletionResponseFormatJsonSchema): The schema of the response
    """

    json_schema: "SchemaCompletionResponseFormatJsonSchema"
    type: SchemaCompletionResponseFormatType = (
        SchemaCompletionResponseFormatType.JSON_SCHEMA
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.schema_completion_response_format_json_schema import (
            SchemaCompletionResponseFormatJsonSchema,
        )

        type = self.type.value

        json_schema = self.json_schema.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "json_schema": json_schema,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.schema_completion_response_format_json_schema import (
            SchemaCompletionResponseFormatJsonSchema,
        )

        d = src_dict.copy()
        type = SchemaCompletionResponseFormatType(d.pop("type"))

        json_schema = SchemaCompletionResponseFormatJsonSchema.from_dict(
            d.pop("json_schema")
        )

        schema_completion_response_format = cls(
            type=type,
            json_schema=json_schema,
        )

        schema_completion_response_format.additional_properties = d
        return schema_completion_response_format

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.tool_type import ToolType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.function_def import FunctionDef


T = TypeVar("T", bound="UpdateToolRequest")


@_attrs_define
class UpdateToolRequest:
    """Payload for updating a tool

    Attributes:
        type (ToolType):  Default: ToolType.FUNCTION.
        name (str): Valid python identifier names
        function (FunctionDef): Function definition
        integration (Union[Unset, Any]):
        system (Union[Unset, Any]):
        api_call (Union[Unset, Any]):
    """

    name: str
    function: "FunctionDef"
    type: ToolType = ToolType.FUNCTION
    integration: Union[Unset, Any] = UNSET
    system: Union[Unset, Any] = UNSET
    api_call: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.function_def import FunctionDef

        type = self.type.value

        name = self.name

        function = self.function.to_dict()

        integration = self.integration

        system = self.system

        api_call = self.api_call

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "name": name,
                "function": function,
            }
        )
        if integration is not UNSET:
            field_dict["integration"] = integration
        if system is not UNSET:
            field_dict["system"] = system
        if api_call is not UNSET:
            field_dict["api_call"] = api_call

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.function_def import FunctionDef

        d = src_dict.copy()
        type = ToolType(d.pop("type"))

        name = d.pop("name")

        function = FunctionDef.from_dict(d.pop("function"))

        integration = d.pop("integration", UNSET)

        system = d.pop("system", UNSET)

        api_call = d.pop("api_call", UNSET)

        update_tool_request = cls(
            type=type,
            name=name,
            function=function,
            integration=integration,
            system=system,
            api_call=api_call,
        )

        update_tool_request.additional_properties = d
        return update_tool_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SleepFor")


@_attrs_define
class SleepFor:
    """
    Attributes:
        seconds (int): The number of seconds to sleep for Default: 0.
        minutes (int): The number of minutes to sleep for Default: 0.
        hours (int): The number of hours to sleep for Default: 0.
        days (int): The number of days to sleep for Default: 0.
    """

    seconds: int = 0
    minutes: int = 0
    hours: int = 0
    days: int = 0
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        seconds = self.seconds

        minutes = self.minutes

        hours = self.hours

        days = self.days

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "seconds": seconds,
                "minutes": minutes,
                "hours": hours,
                "days": days,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        seconds = d.pop("seconds")

        minutes = d.pop("minutes")

        hours = d.pop("hours")

        days = d.pop("days")

        sleep_for = cls(
            seconds=seconds,
            minutes=minutes,
            hours=hours,
            days=days,
        )

        sleep_for.additional_properties = d
        return sleep_for

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.hybrid_doc_search_request_lang import HybridDocSearchRequestLang
from ..types import UNSET, Unset

T = TypeVar("T", bound="HybridDocSearchRequest")


@_attrs_define
class HybridDocSearchRequest:
    """
    Attributes:
        limit (int):  Default: 10.
        lang (HybridDocSearchRequestLang): The language to be used for text-only search. Support for other languages
            coming soon. Default: HybridDocSearchRequestLang.EN_US.
        confidence (float): The confidence cutoff level Default: 0.5.
        alpha (float): The weight to apply to BM25 vs Vector search results. 0 => pure BM25; 1 => pure vector; Default:
            0.75.
        text (str): Text to use in the search. In `hybrid` search mode, either `text` or both `text` and `vector` fields
            are required.
        vector (List[float]): Vector to use in the search. Must be the same dimensions as the embedding model or else an
            error will be thrown.
    """

    text: str
    vector: List[float]
    limit: int = 10
    lang: HybridDocSearchRequestLang = HybridDocSearchRequestLang.EN_US
    confidence: float = 0.5
    alpha: float = 0.75
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        limit = self.limit

        lang = self.lang.value

        confidence = self.confidence

        alpha = self.alpha

        text = self.text

        vector = self.vector

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "limit": limit,
                "lang": lang,
                "confidence": confidence,
                "alpha": alpha,
                "text": text,
                "vector": vector,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        limit = d.pop("limit")

        lang = HybridDocSearchRequestLang(d.pop("lang"))

        confidence = d.pop("confidence")

        alpha = d.pop("alpha")

        text = d.pop("text")

        vector = cast(List[float], d.pop("vector"))

        hybrid_doc_search_request = cls(
            limit=limit,
            lang=lang,
            confidence=confidence,
            alpha=alpha,
            text=text,
            vector=vector,
        )

        hybrid_doc_search_request.additional_properties = d
        return hybrid_doc_search_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .agent import Agent
from .agent_docs_route_list_direction import AgentDocsRouteListDirection
from .agent_docs_route_list_response_200 import AgentDocsRouteListResponse200
from .agent_docs_route_list_sort_by import AgentDocsRouteListSortBy
from .agent_metadata import AgentMetadata
from .agent_tools_route_list_direction import AgentToolsRouteListDirection
from .agent_tools_route_list_response_200 import AgentToolsRouteListResponse200
from .agent_tools_route_list_sort_by import AgentToolsRouteListSortBy
from .base_agent import BaseAgent
from .base_agent_metadata import BaseAgentMetadata
from .base_chat_output import BaseChatOutput
from .base_entry_content_type_0_item_type_0 import BaseEntryContentType0ItemType0
from .base_entry_content_type_0_item_type_0_type import (
    BaseEntryContentType0ItemType0Type,
)
from .base_entry_content_type_0_item_type_1 import BaseEntryContentType0ItemType1
from .base_entry_content_type_0_item_type_1_image_url import (
    BaseEntryContentType0ItemType1ImageUrl,
)
from .base_entry_content_type_0_item_type_1_type import (
    BaseEntryContentType0ItemType1Type,
)
from .base_entry_content_type_8_item_type_0_item_type_0 import (
    BaseEntryContentType8ItemType0ItemType0,
)
from .base_entry_content_type_8_item_type_0_item_type_0_type import (
    BaseEntryContentType8ItemType0ItemType0Type,
)
from .base_entry_content_type_8_item_type_0_item_type_1 import (
    BaseEntryContentType8ItemType0ItemType1,
)
from .base_entry_content_type_8_item_type_0_item_type_1_image_url import (
    BaseEntryContentType8ItemType0ItemType1ImageUrl,
)
from .base_entry_content_type_8_item_type_0_item_type_1_type import (
    BaseEntryContentType8ItemType0ItemType1Type,
)
from .base_entry_source import BaseEntrySource
from .base_token_log_prob import BaseTokenLogProb
from .case_then import CaseThen
from .case_then_case_type_1 import CaseThenCaseType1
from .chat_input import ChatInput
from .chat_input_logit_bias import ChatInputLogitBias
from .chat_input_messages_item import ChatInputMessagesItem
from .chat_input_messages_item_content_type_2_item_type_0 import (
    ChatInputMessagesItemContentType2ItemType0,
)
from .chat_input_messages_item_content_type_2_item_type_0_type import (
    ChatInputMessagesItemContentType2ItemType0Type,
)
from .chat_input_messages_item_content_type_2_item_type_1 import (
    ChatInputMessagesItemContentType2ItemType1,
)
from .chat_input_messages_item_content_type_2_item_type_1_image_url import (
    ChatInputMessagesItemContentType2ItemType1ImageUrl,
)
from .chat_input_messages_item_content_type_2_item_type_1_type import (
    ChatInputMessagesItemContentType2ItemType1Type,
)
from .chat_input_tool_choice_type_0 import ChatInputToolChoiceType0
from .chat_output_chunk import ChatOutputChunk
from .chat_output_chunk_delta import ChatOutputChunkDelta
from .chat_output_chunk_delta_content_type_2_item_type_0 import (
    ChatOutputChunkDeltaContentType2ItemType0,
)
from .chat_output_chunk_delta_content_type_2_item_type_0_type import (
    ChatOutputChunkDeltaContentType2ItemType0Type,
)
from .chat_output_chunk_delta_content_type_2_item_type_1 import (
    ChatOutputChunkDeltaContentType2ItemType1,
)
from .chat_output_chunk_delta_content_type_2_item_type_1_image_url import (
    ChatOutputChunkDeltaContentType2ItemType1ImageUrl,
)
from .chat_output_chunk_delta_content_type_2_item_type_1_type import (
    ChatOutputChunkDeltaContentType2ItemType1Type,
)
from .chat_settings import ChatSettings
from .chat_settings_logit_bias import ChatSettingsLogitBias
from .chat_token_log_prob import ChatTokenLogProb
from .chosen_api_call import ChosenApiCall
from .chosen_api_call_type import ChosenApiCallType
from .chosen_function_call import ChosenFunctionCall
from .chosen_function_call_type import ChosenFunctionCallType
from .chosen_integration_call import ChosenIntegrationCall
from .chosen_integration_call_type import ChosenIntegrationCallType
from .chosen_system_call import ChosenSystemCall
from .chosen_system_call_type import ChosenSystemCallType
from .chunk_chat_response import ChunkChatResponse
from .competion_usage import CompetionUsage
from .context_overflow_type import ContextOverflowType
from .create_agent_request import CreateAgentRequest
from .create_agent_request_metadata import CreateAgentRequestMetadata
from .create_doc_request import CreateDocRequest
from .create_doc_request_metadata import CreateDocRequestMetadata
from .create_execution_request import CreateExecutionRequest
from .create_execution_request_input import CreateExecutionRequestInput
from .create_execution_request_metadata import CreateExecutionRequestMetadata
from .create_session_request import CreateSessionRequest
from .create_session_request_metadata import CreateSessionRequestMetadata
from .create_task_request import CreateTaskRequest
from .create_task_request_additional_property_item_type_17 import (
    CreateTaskRequestAdditionalPropertyItemType17,
)
from .create_task_request_additional_property_item_type_17_kind import (
    CreateTaskRequestAdditionalPropertyItemType17Kind,
)
from .create_task_request_input_schema_type_0 import CreateTaskRequestInputSchemaType0
from .create_task_request_main_item_type_17 import CreateTaskRequestMainItemType17
from .create_task_request_main_item_type_17_kind import (
    CreateTaskRequestMainItemType17Kind,
)
from .create_task_request_metadata import CreateTaskRequestMetadata
from .create_user_request import CreateUserRequest
from .create_user_request_metadata import CreateUserRequestMetadata
from .default_chat_settings import DefaultChatSettings
from .doc import Doc
from .doc_metadata import DocMetadata
from .doc_owner import DocOwner
from .doc_owner_role import DocOwnerRole
from .doc_reference import DocReference
from .doc_search_response import DocSearchResponse
from .docs_search_route_search_body import DocsSearchRouteSearchBody
from .embed_query_request import EmbedQueryRequest
from .embed_query_response import EmbedQueryResponse
from .embed_step import EmbedStep
from .embed_step_kind import EmbedStepKind
from .entries_chat_ml_role import EntriesChatMLRole
from .entries_image_detail import EntriesImageDetail
from .entries_relation import EntriesRelation
from .error_workflow_step import ErrorWorkflowStep
from .error_workflow_step_kind import ErrorWorkflowStepKind
from .evaluate_step import EvaluateStep
from .evaluate_step_evaluate import EvaluateStepEvaluate
from .evaluate_step_kind import EvaluateStepKind
from .execution import Execution
from .execution_input import ExecutionInput
from .execution_metadata import ExecutionMetadata
from .execution_status import ExecutionStatus
from .finish_reason import FinishReason
from .foreach_do import ForeachDo
from .foreach_step import ForeachStep
from .foreach_step_kind import ForeachStepKind
from .function_call_option import FunctionCallOption
from .function_def import FunctionDef
from .function_def_parameters import FunctionDefParameters
from .get_step import GetStep
from .get_step_kind import GetStepKind
from .hybrid_doc_search_request import HybridDocSearchRequest
from .hybrid_doc_search_request_lang import HybridDocSearchRequestLang
from .if_else_workflow_step import IfElseWorkflowStep
from .if_else_workflow_step_kind import IfElseWorkflowStepKind
from .job_state import JobState
from .job_status import JobStatus
from .log_prob_response import LogProbResponse
from .log_step import LogStep
from .log_step_kind import LogStepKind
from .message_chat_response import MessageChatResponse
from .named_api_call_choice import NamedApiCallChoice
from .named_function_choice import NamedFunctionChoice
from .named_integration_choice import NamedIntegrationChoice
from .named_system_choice import NamedSystemChoice
from .parallel_step import ParallelStep
from .parallel_step_kind import ParallelStepKind
from .patch_agent_request import PatchAgentRequest
from .patch_agent_request_metadata import PatchAgentRequestMetadata
from .patch_session_request import PatchSessionRequest
from .patch_session_request_metadata import PatchSessionRequestMetadata
from .patch_task_request import PatchTaskRequest
from .patch_task_request_additional_property_item_type_17 import (
    PatchTaskRequestAdditionalPropertyItemType17,
)
from .patch_task_request_input_schema_type_0 import PatchTaskRequestInputSchemaType0
from .patch_task_request_main_item_type_17 import PatchTaskRequestMainItemType17
from .patch_task_request_metadata import PatchTaskRequestMetadata
from .patch_tool_request import PatchToolRequest
from .patch_user_request import PatchUserRequest
from .patch_user_request_metadata import PatchUserRequestMetadata
from .prompt_step import PromptStep
from .prompt_step_kind import PromptStepKind
from .prompt_step_prompt_type_1_item import PromptStepPromptType1Item
from .prompt_step_prompt_type_1_item_content_type_2_item_type_0 import (
    PromptStepPromptType1ItemContentType2ItemType0,
)
from .prompt_step_prompt_type_1_item_content_type_2_item_type_0_type import (
    PromptStepPromptType1ItemContentType2ItemType0Type,
)
from .prompt_step_prompt_type_1_item_content_type_2_item_type_1 import (
    PromptStepPromptType1ItemContentType2ItemType1,
)
from .prompt_step_prompt_type_1_item_content_type_2_item_type_1_image_url import (
    PromptStepPromptType1ItemContentType2ItemType1ImageUrl,
)
from .prompt_step_prompt_type_1_item_content_type_2_item_type_1_type import (
    PromptStepPromptType1ItemContentType2ItemType1Type,
)
from .resource_created_response import ResourceCreatedResponse
from .resource_deleted_response import ResourceDeletedResponse
from .resource_updated_response import ResourceUpdatedResponse
from .return_step import ReturnStep
from .return_step_kind import ReturnStepKind
from .return_step_return import ReturnStepReturn
from .route_embed_body import RouteEmbedBody
from .route_list_direction import RouteListDirection
from .route_list_response_200 import RouteListResponse200
from .route_list_sort_by import RouteListSortBy
from .schema_completion_response_format import SchemaCompletionResponseFormat
from .schema_completion_response_format_json_schema import (
    SchemaCompletionResponseFormatJsonSchema,
)
from .schema_completion_response_format_type import SchemaCompletionResponseFormatType
from .search_step import SearchStep
from .search_step_kind import SearchStepKind
from .set_step import SetStep
from .set_step_kind import SetStepKind
from .set_step_set import SetStepSet
from .simple_completion_response_format import SimpleCompletionResponseFormat
from .simple_completion_response_format_type import SimpleCompletionResponseFormatType
from .sleep_for import SleepFor
from .sleep_step import SleepStep
from .sleep_step_kind import SleepStepKind
from .snippet import Snippet
from .switch_step import SwitchStep
from .switch_step_kind import SwitchStepKind
from .task import Task
from .task_additional_property_item_type_17 import TaskAdditionalPropertyItemType17
from .task_additional_property_item_type_17_kind import (
    TaskAdditionalPropertyItemType17Kind,
)
from .task_executions_route_list_direction import TaskExecutionsRouteListDirection
from .task_executions_route_list_response_200 import TaskExecutionsRouteListResponse200
from .task_executions_route_list_sort_by import TaskExecutionsRouteListSortBy
from .task_input_schema_type_0 import TaskInputSchemaType0
from .task_main_item_type_17 import TaskMainItemType17
from .task_main_item_type_17_kind import TaskMainItemType17Kind
from .task_metadata import TaskMetadata
from .task_token_resume_execution_request import TaskTokenResumeExecutionRequest
from .task_token_resume_execution_request_input import (
    TaskTokenResumeExecutionRequestInput,
)
from .task_token_resume_execution_request_status import (
    TaskTokenResumeExecutionRequestStatus,
)
from .task_tool import TaskTool
from .tasks_create_or_update_route_create_or_update_accept import (
    TasksCreateOrUpdateRouteCreateOrUpdateAccept,
)
from .tasks_route_create_accept import TasksRouteCreateAccept
from .text_only_doc_search_request import TextOnlyDocSearchRequest
from .text_only_doc_search_request_lang import TextOnlyDocSearchRequestLang
from .tool import Tool
from .tool_call_step import ToolCallStep
from .tool_call_step_arguments_type_0 import ToolCallStepArgumentsType0
from .tool_call_step_arguments_type_1 import ToolCallStepArgumentsType1
from .tool_call_step_kind import ToolCallStepKind
from .tool_response import ToolResponse
from .tool_response_output import ToolResponseOutput
from .tool_type import ToolType
from .transition import Transition
from .transition_event import TransitionEvent
from .transition_event_type import TransitionEventType
from .transition_metadata import TransitionMetadata
from .transition_target import TransitionTarget
from .transition_type import TransitionType
from .update_agent_request import UpdateAgentRequest
from .update_agent_request_metadata import UpdateAgentRequestMetadata
from .update_execution_request import UpdateExecutionRequest
from .update_execution_request_status import UpdateExecutionRequestStatus
from .update_session_request import UpdateSessionRequest
from .update_session_request_metadata import UpdateSessionRequestMetadata
from .update_task_request import UpdateTaskRequest
from .update_task_request_additional_property_item_type_17 import (
    UpdateTaskRequestAdditionalPropertyItemType17,
)
from .update_task_request_additional_property_item_type_17_kind import (
    UpdateTaskRequestAdditionalPropertyItemType17Kind,
)
from .update_task_request_input_schema_type_0 import UpdateTaskRequestInputSchemaType0
from .update_task_request_main_item_type_17 import UpdateTaskRequestMainItemType17
from .update_task_request_main_item_type_17_kind import (
    UpdateTaskRequestMainItemType17Kind,
)
from .update_task_request_metadata import UpdateTaskRequestMetadata
from .update_tool_request import UpdateToolRequest
from .update_user_request import UpdateUserRequest
from .update_user_request_metadata import UpdateUserRequestMetadata
from .user import User
from .user_docs_route_list_direction import UserDocsRouteListDirection
from .user_docs_route_list_response_200 import UserDocsRouteListResponse200
from .user_docs_route_list_sort_by import UserDocsRouteListSortBy
from .user_docs_search_route_search_body import UserDocsSearchRouteSearchBody
from .user_metadata import UserMetadata
from .vector_doc_search_request import VectorDocSearchRequest
from .vector_doc_search_request_lang import VectorDocSearchRequestLang
from .wait_for_input_info import WaitForInputInfo
from .wait_for_input_info_info import WaitForInputInfoInfo
from .wait_for_input_step import WaitForInputStep
from .wait_for_input_step_kind import WaitForInputStepKind
from .yield_step import YieldStep
from .yield_step_arguments_type_0 import YieldStepArgumentsType0
from .yield_step_arguments_type_1 import YieldStepArgumentsType1
from .yield_step_kind import YieldStepKind

__all__ = (
    "Agent",
    "AgentDocsRouteListDirection",
    "AgentDocsRouteListResponse200",
    "AgentDocsRouteListSortBy",
    "AgentMetadata",
    "AgentToolsRouteListDirection",
    "AgentToolsRouteListResponse200",
    "AgentToolsRouteListSortBy",
    "BaseAgent",
    "BaseAgentMetadata",
    "BaseChatOutput",
    "BaseEntryContentType0ItemType0",
    "BaseEntryContentType0ItemType0Type",
    "BaseEntryContentType0ItemType1",
    "BaseEntryContentType0ItemType1ImageUrl",
    "BaseEntryContentType0ItemType1Type",
    "BaseEntryContentType8ItemType0ItemType0",
    "BaseEntryContentType8ItemType0ItemType0Type",
    "BaseEntryContentType8ItemType0ItemType1",
    "BaseEntryContentType8ItemType0ItemType1ImageUrl",
    "BaseEntryContentType8ItemType0ItemType1Type",
    "BaseEntrySource",
    "BaseTokenLogProb",
    "CaseThen",
    "CaseThenCaseType1",
    "ChatInput",
    "ChatInputLogitBias",
    "ChatInputMessagesItem",
    "ChatInputMessagesItemContentType2ItemType0",
    "ChatInputMessagesItemContentType2ItemType0Type",
    "ChatInputMessagesItemContentType2ItemType1",
    "ChatInputMessagesItemContentType2ItemType1ImageUrl",
    "ChatInputMessagesItemContentType2ItemType1Type",
    "ChatInputToolChoiceType0",
    "ChatOutputChunk",
    "ChatOutputChunkDelta",
    "ChatOutputChunkDeltaContentType2ItemType0",
    "ChatOutputChunkDeltaContentType2ItemType0Type",
    "ChatOutputChunkDeltaContentType2ItemType1",
    "ChatOutputChunkDeltaContentType2ItemType1ImageUrl",
    "ChatOutputChunkDeltaContentType2ItemType1Type",
    "ChatSettings",
    "ChatSettingsLogitBias",
    "ChatTokenLogProb",
    "ChosenApiCall",
    "ChosenApiCallType",
    "ChosenFunctionCall",
    "ChosenFunctionCallType",
    "ChosenIntegrationCall",
    "ChosenIntegrationCallType",
    "ChosenSystemCall",
    "ChosenSystemCallType",
    "ChunkChatResponse",
    "CompetionUsage",
    "ContextOverflowType",
    "CreateAgentRequest",
    "CreateAgentRequestMetadata",
    "CreateDocRequest",
    "CreateDocRequestMetadata",
    "CreateExecutionRequest",
    "CreateExecutionRequestInput",
    "CreateExecutionRequestMetadata",
    "CreateSessionRequest",
    "CreateSessionRequestMetadata",
    "CreateTaskRequest",
    "CreateTaskRequestAdditionalPropertyItemType17",
    "CreateTaskRequestAdditionalPropertyItemType17Kind",
    "CreateTaskRequestInputSchemaType0",
    "CreateTaskRequestMainItemType17",
    "CreateTaskRequestMainItemType17Kind",
    "CreateTaskRequestMetadata",
    "CreateUserRequest",
    "CreateUserRequestMetadata",
    "DefaultChatSettings",
    "Doc",
    "DocMetadata",
    "DocOwner",
    "DocOwnerRole",
    "DocReference",
    "DocSearchResponse",
    "DocsSearchRouteSearchBody",
    "EmbedQueryRequest",
    "EmbedQueryResponse",
    "EmbedStep",
    "EmbedStepKind",
    "EntriesChatMLRole",
    "EntriesImageDetail",
    "EntriesRelation",
    "ErrorWorkflowStep",
    "ErrorWorkflowStepKind",
    "EvaluateStep",
    "EvaluateStepEvaluate",
    "EvaluateStepKind",
    "Execution",
    "ExecutionInput",
    "ExecutionMetadata",
    "ExecutionStatus",
    "FinishReason",
    "ForeachDo",
    "ForeachStep",
    "ForeachStepKind",
    "FunctionCallOption",
    "FunctionDef",
    "FunctionDefParameters",
    "GetStep",
    "GetStepKind",
    "HybridDocSearchRequest",
    "HybridDocSearchRequestLang",
    "IfElseWorkflowStep",
    "IfElseWorkflowStepKind",
    "JobState",
    "JobStatus",
    "LogProbResponse",
    "LogStep",
    "LogStepKind",
    "MessageChatResponse",
    "NamedApiCallChoice",
    "NamedFunctionChoice",
    "NamedIntegrationChoice",
    "NamedSystemChoice",
    "ParallelStep",
    "ParallelStepKind",
    "PatchAgentRequest",
    "PatchAgentRequestMetadata",
    "PatchSessionRequest",
    "PatchSessionRequestMetadata",
    "PatchTaskRequest",
    "PatchTaskRequestAdditionalPropertyItemType17",
    "PatchTaskRequestInputSchemaType0",
    "PatchTaskRequestMainItemType17",
    "PatchTaskRequestMetadata",
    "PatchToolRequest",
    "PatchUserRequest",
    "PatchUserRequestMetadata",
    "PromptStep",
    "PromptStepKind",
    "PromptStepPromptType1Item",
    "PromptStepPromptType1ItemContentType2ItemType0",
    "PromptStepPromptType1ItemContentType2ItemType0Type",
    "PromptStepPromptType1ItemContentType2ItemType1",
    "PromptStepPromptType1ItemContentType2ItemType1ImageUrl",
    "PromptStepPromptType1ItemContentType2ItemType1Type",
    "ResourceCreatedResponse",
    "ResourceDeletedResponse",
    "ResourceUpdatedResponse",
    "ReturnStep",
    "ReturnStepKind",
    "ReturnStepReturn",
    "RouteEmbedBody",
    "RouteListDirection",
    "RouteListResponse200",
    "RouteListSortBy",
    "SchemaCompletionResponseFormat",
    "SchemaCompletionResponseFormatJsonSchema",
    "SchemaCompletionResponseFormatType",
    "SearchStep",
    "SearchStepKind",
    "SetStep",
    "SetStepKind",
    "SetStepSet",
    "SimpleCompletionResponseFormat",
    "SimpleCompletionResponseFormatType",
    "SleepFor",
    "SleepStep",
    "SleepStepKind",
    "Snippet",
    "SwitchStep",
    "SwitchStepKind",
    "Task",
    "TaskAdditionalPropertyItemType17",
    "TaskAdditionalPropertyItemType17Kind",
    "TaskExecutionsRouteListDirection",
    "TaskExecutionsRouteListResponse200",
    "TaskExecutionsRouteListSortBy",
    "TaskInputSchemaType0",
    "TaskMainItemType17",
    "TaskMainItemType17Kind",
    "TaskMetadata",
    "TasksCreateOrUpdateRouteCreateOrUpdateAccept",
    "TasksRouteCreateAccept",
    "TaskTokenResumeExecutionRequest",
    "TaskTokenResumeExecutionRequestInput",
    "TaskTokenResumeExecutionRequestStatus",
    "TaskTool",
    "TextOnlyDocSearchRequest",
    "TextOnlyDocSearchRequestLang",
    "Tool",
    "ToolCallStep",
    "ToolCallStepArgumentsType0",
    "ToolCallStepArgumentsType1",
    "ToolCallStepKind",
    "ToolResponse",
    "ToolResponseOutput",
    "ToolType",
    "Transition",
    "TransitionEvent",
    "TransitionEventType",
    "TransitionMetadata",
    "TransitionTarget",
    "TransitionType",
    "UpdateAgentRequest",
    "UpdateAgentRequestMetadata",
    "UpdateExecutionRequest",
    "UpdateExecutionRequestStatus",
    "UpdateSessionRequest",
    "UpdateSessionRequestMetadata",
    "UpdateTaskRequest",
    "UpdateTaskRequestAdditionalPropertyItemType17",
    "UpdateTaskRequestAdditionalPropertyItemType17Kind",
    "UpdateTaskRequestInputSchemaType0",
    "UpdateTaskRequestMainItemType17",
    "UpdateTaskRequestMainItemType17Kind",
    "UpdateTaskRequestMetadata",
    "UpdateToolRequest",
    "UpdateUserRequest",
    "UpdateUserRequestMetadata",
    "User",
    "UserDocsRouteListDirection",
    "UserDocsRouteListResponse200",
    "UserDocsRouteListSortBy",
    "UserDocsSearchRouteSearchBody",
    "UserMetadata",
    "VectorDocSearchRequest",
    "VectorDocSearchRequestLang",
    "WaitForInputInfo",
    "WaitForInputInfoInfo",
    "WaitForInputStep",
    "WaitForInputStepKind",
    "YieldStep",
    "YieldStepArgumentsType0",
    "YieldStepArgumentsType1",
    "YieldStepKind",
)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embed_step import EmbedStep
    from ..models.error_workflow_step import ErrorWorkflowStep
    from ..models.evaluate_step import EvaluateStep
    from ..models.foreach_step import ForeachStep
    from ..models.get_step import GetStep
    from ..models.if_else_workflow_step import IfElseWorkflowStep
    from ..models.log_step import LogStep
    from ..models.parallel_step import ParallelStep
    from ..models.patch_task_request_additional_property_item_type_17 import (
        PatchTaskRequestAdditionalPropertyItemType17,
    )
    from ..models.patch_task_request_input_schema_type_0 import (
        PatchTaskRequestInputSchemaType0,
    )
    from ..models.patch_task_request_main_item_type_17 import (
        PatchTaskRequestMainItemType17,
    )
    from ..models.patch_task_request_metadata import PatchTaskRequestMetadata
    from ..models.prompt_step import PromptStep
    from ..models.return_step import ReturnStep
    from ..models.search_step import SearchStep
    from ..models.set_step import SetStep
    from ..models.sleep_step import SleepStep
    from ..models.switch_step import SwitchStep
    from ..models.task_tool import TaskTool
    from ..models.tool_call_step import ToolCallStep
    from ..models.wait_for_input_step import WaitForInputStep
    from ..models.yield_step import YieldStep


T = TypeVar("T", bound="PatchTaskRequest")


@_attrs_define
class PatchTaskRequest:
    """Payload for patching a task

    Attributes:
        description (Union[Unset, str]):  Default: ''.
        main (Union[Unset, List[Union['EmbedStep', 'ErrorWorkflowStep', 'EvaluateStep', 'ForeachStep', 'GetStep',
            'IfElseWorkflowStep', 'LogStep', 'ParallelStep', 'PatchTaskRequestMainItemType17', 'PromptStep', 'ReturnStep',
            'SearchStep', 'SetStep', 'SleepStep', 'SwitchStep', 'ToolCallStep', 'WaitForInputStep', 'YieldStep']]]): The
            entrypoint of the task.
        input_schema (Union['PatchTaskRequestInputSchemaType0', None, Unset]): The schema for the input to the task.
            `null` means all inputs are valid.
        tools (Union[Unset, List['TaskTool']]): Tools defined specifically for this task not included in the Agent
            itself.
        inherit_tools (Union[Unset, bool]): Whether to inherit tools from the parent agent or not. Defaults to true.
            Default: True.
        metadata (Union[Unset, PatchTaskRequestMetadata]):
    """

    description: Union[Unset, str] = ""
    main: Union[
        Unset,
        List[
            Union[
                "EmbedStep",
                "ErrorWorkflowStep",
                "EvaluateStep",
                "ForeachStep",
                "GetStep",
                "IfElseWorkflowStep",
                "LogStep",
                "ParallelStep",
                "PatchTaskRequestMainItemType17",
                "PromptStep",
                "ReturnStep",
                "SearchStep",
                "SetStep",
                "SleepStep",
                "SwitchStep",
                "ToolCallStep",
                "WaitForInputStep",
                "YieldStep",
            ]
        ],
    ] = UNSET
    input_schema: Union["PatchTaskRequestInputSchemaType0", None, Unset] = UNSET
    tools: Union[Unset, List["TaskTool"]] = UNSET
    inherit_tools: Union[Unset, bool] = True
    metadata: Union[Unset, "PatchTaskRequestMetadata"] = UNSET
    additional_properties: Dict[
        str,
        List[
            Union[
                "EmbedStep",
                "ErrorWorkflowStep",
                "EvaluateStep",
                "ForeachStep",
                "GetStep",
                "IfElseWorkflowStep",
                "LogStep",
                "ParallelStep",
                "PatchTaskRequestAdditionalPropertyItemType17",
                "PromptStep",
                "ReturnStep",
                "SearchStep",
                "SetStep",
                "SleepStep",
                "SwitchStep",
                "ToolCallStep",
                "WaitForInputStep",
                "YieldStep",
            ]
        ],
    ] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.embed_step import EmbedStep
        from ..models.error_workflow_step import ErrorWorkflowStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.foreach_step import ForeachStep
        from ..models.get_step import GetStep
        from ..models.if_else_workflow_step import IfElseWorkflowStep
        from ..models.log_step import LogStep
        from ..models.parallel_step import ParallelStep
        from ..models.patch_task_request_additional_property_item_type_17 import (
            PatchTaskRequestAdditionalPropertyItemType17,
        )
        from ..models.patch_task_request_input_schema_type_0 import (
            PatchTaskRequestInputSchemaType0,
        )
        from ..models.patch_task_request_main_item_type_17 import (
            PatchTaskRequestMainItemType17,
        )
        from ..models.patch_task_request_metadata import PatchTaskRequestMetadata
        from ..models.prompt_step import PromptStep
        from ..models.return_step import ReturnStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.sleep_step import SleepStep
        from ..models.switch_step import SwitchStep
        from ..models.task_tool import TaskTool
        from ..models.tool_call_step import ToolCallStep
        from ..models.wait_for_input_step import WaitForInputStep
        from ..models.yield_step import YieldStep

        description = self.description

        main: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.main, Unset):
            main = []
            for main_item_data in self.main:
                main_item: Dict[str, Any]
                if isinstance(main_item_data, EvaluateStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, ToolCallStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, PromptStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, GetStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, SetStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, LogStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, EmbedStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, SearchStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, ReturnStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, SleepStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, ErrorWorkflowStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, YieldStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, WaitForInputStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, IfElseWorkflowStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, SwitchStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, ForeachStep):
                    main_item = main_item_data.to_dict()
                elif isinstance(main_item_data, ParallelStep):
                    main_item = main_item_data.to_dict()
                else:
                    main_item = main_item_data.to_dict()

                main.append(main_item)

        input_schema: Union[Dict[str, Any], None, Unset]
        if isinstance(self.input_schema, Unset):
            input_schema = UNSET
        elif isinstance(self.input_schema, PatchTaskRequestInputSchemaType0):
            input_schema = self.input_schema.to_dict()
        else:
            input_schema = self.input_schema

        tools: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tools, Unset):
            tools = []
            for tools_item_data in self.tools:
                tools_item = tools_item_data.to_dict()
                tools.append(tools_item)

        inherit_tools = self.inherit_tools

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = []
            for additional_property_item_data in prop:
                additional_property_item: Dict[str, Any]
                if isinstance(additional_property_item_data, EvaluateStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, ToolCallStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, PromptStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, GetStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, SetStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, LogStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, EmbedStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, SearchStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, ReturnStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, SleepStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, ErrorWorkflowStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, YieldStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, WaitForInputStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, IfElseWorkflowStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, SwitchStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, ForeachStep):
                    additional_property_item = additional_property_item_data.to_dict()
                elif isinstance(additional_property_item_data, ParallelStep):
                    additional_property_item = additional_property_item_data.to_dict()
                else:
                    additional_property_item = additional_property_item_data.to_dict()

                field_dict[prop_name].append(additional_property_item)

        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if main is not UNSET:
            field_dict["main"] = main
        if input_schema is not UNSET:
            field_dict["input_schema"] = input_schema
        if tools is not UNSET:
            field_dict["tools"] = tools
        if inherit_tools is not UNSET:
            field_dict["inherit_tools"] = inherit_tools
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embed_step import EmbedStep
        from ..models.error_workflow_step import ErrorWorkflowStep
        from ..models.evaluate_step import EvaluateStep
        from ..models.foreach_step import ForeachStep
        from ..models.get_step import GetStep
        from ..models.if_else_workflow_step import IfElseWorkflowStep
        from ..models.log_step import LogStep
        from ..models.parallel_step import ParallelStep
        from ..models.patch_task_request_additional_property_item_type_17 import (
            PatchTaskRequestAdditionalPropertyItemType17,
        )
        from ..models.patch_task_request_input_schema_type_0 import (
            PatchTaskRequestInputSchemaType0,
        )
        from ..models.patch_task_request_main_item_type_17 import (
            PatchTaskRequestMainItemType17,
        )
        from ..models.patch_task_request_metadata import PatchTaskRequestMetadata
        from ..models.prompt_step import PromptStep
        from ..models.return_step import ReturnStep
        from ..models.search_step import SearchStep
        from ..models.set_step import SetStep
        from ..models.sleep_step import SleepStep
        from ..models.switch_step import SwitchStep
        from ..models.task_tool import TaskTool
        from ..models.tool_call_step import ToolCallStep
        from ..models.wait_for_input_step import WaitForInputStep
        from ..models.yield_step import YieldStep

        d = src_dict.copy()
        description = d.pop("description", UNSET)

        main = []
        _main = d.pop("main", UNSET)
        for main_item_data in _main or []:

            def _parse_main_item(
                data: object,
            ) -> Union[
                "EmbedStep",
                "ErrorWorkflowStep",
                "EvaluateStep",
                "ForeachStep",
                "GetStep",
                "IfElseWorkflowStep",
                "LogStep",
                "ParallelStep",
                "PatchTaskRequestMainItemType17",
                "PromptStep",
                "ReturnStep",
                "SearchStep",
                "SetStep",
                "SleepStep",
                "SwitchStep",
                "ToolCallStep",
                "WaitForInputStep",
                "YieldStep",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_0 = EvaluateStep.from_dict(data)

                    return main_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_1 = ToolCallStep.from_dict(data)

                    return main_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_2 = PromptStep.from_dict(data)

                    return main_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_3 = GetStep.from_dict(data)

                    return main_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_4 = SetStep.from_dict(data)

                    return main_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_5 = LogStep.from_dict(data)

                    return main_item_type_5
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_6 = EmbedStep.from_dict(data)

                    return main_item_type_6
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_7 = SearchStep.from_dict(data)

                    return main_item_type_7
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_8 = ReturnStep.from_dict(data)

                    return main_item_type_8
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_9 = SleepStep.from_dict(data)

                    return main_item_type_9
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_10 = ErrorWorkflowStep.from_dict(data)

                    return main_item_type_10
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_11 = YieldStep.from_dict(data)

                    return main_item_type_11
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_12 = WaitForInputStep.from_dict(data)

                    return main_item_type_12
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_13 = IfElseWorkflowStep.from_dict(data)

                    return main_item_type_13
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_14 = SwitchStep.from_dict(data)

                    return main_item_type_14
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_15 = ForeachStep.from_dict(data)

                    return main_item_type_15
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    main_item_type_16 = ParallelStep.from_dict(data)

                    return main_item_type_16
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                main_item_type_17 = PatchTaskRequestMainItemType17.from_dict(data)

                return main_item_type_17

            main_item = _parse_main_item(main_item_data)

            main.append(main_item)

        def _parse_input_schema(
            data: object,
        ) -> Union["PatchTaskRequestInputSchemaType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_schema_type_0 = PatchTaskRequestInputSchemaType0.from_dict(data)

                return input_schema_type_0
            except:  # noqa: E722
                pass
            return cast(Union["PatchTaskRequestInputSchemaType0", None, Unset], data)

        input_schema = _parse_input_schema(d.pop("input_schema", UNSET))

        tools = []
        _tools = d.pop("tools", UNSET)
        for tools_item_data in _tools or []:
            tools_item = TaskTool.from_dict(tools_item_data)

            tools.append(tools_item)

        inherit_tools = d.pop("inherit_tools", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, PatchTaskRequestMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = PatchTaskRequestMetadata.from_dict(_metadata)

        patch_task_request = cls(
            description=description,
            main=main,
            input_schema=input_schema,
            tools=tools,
            inherit_tools=inherit_tools,
            metadata=metadata,
        )

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = []
            _additional_property = prop_dict
            for additional_property_item_data in _additional_property:

                def _parse_additional_property_item(
                    data: object,
                ) -> Union[
                    "EmbedStep",
                    "ErrorWorkflowStep",
                    "EvaluateStep",
                    "ForeachStep",
                    "GetStep",
                    "IfElseWorkflowStep",
                    "LogStep",
                    "ParallelStep",
                    "PatchTaskRequestAdditionalPropertyItemType17",
                    "PromptStep",
                    "ReturnStep",
                    "SearchStep",
                    "SetStep",
                    "SleepStep",
                    "SwitchStep",
                    "ToolCallStep",
                    "WaitForInputStep",
                    "YieldStep",
                ]:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_0 = EvaluateStep.from_dict(data)

                        return additional_property_item_type_0
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_1 = ToolCallStep.from_dict(data)

                        return additional_property_item_type_1
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_2 = PromptStep.from_dict(data)

                        return additional_property_item_type_2
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_3 = GetStep.from_dict(data)

                        return additional_property_item_type_3
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_4 = SetStep.from_dict(data)

                        return additional_property_item_type_4
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_5 = LogStep.from_dict(data)

                        return additional_property_item_type_5
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_6 = EmbedStep.from_dict(data)

                        return additional_property_item_type_6
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_7 = SearchStep.from_dict(data)

                        return additional_property_item_type_7
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_8 = ReturnStep.from_dict(data)

                        return additional_property_item_type_8
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_9 = SleepStep.from_dict(data)

                        return additional_property_item_type_9
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_10 = ErrorWorkflowStep.from_dict(
                            data
                        )

                        return additional_property_item_type_10
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_11 = YieldStep.from_dict(data)

                        return additional_property_item_type_11
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_12 = WaitForInputStep.from_dict(
                            data
                        )

                        return additional_property_item_type_12
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_13 = IfElseWorkflowStep.from_dict(
                            data
                        )

                        return additional_property_item_type_13
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_14 = SwitchStep.from_dict(data)

                        return additional_property_item_type_14
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_15 = ForeachStep.from_dict(data)

                        return additional_property_item_type_15
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        additional_property_item_type_16 = ParallelStep.from_dict(data)

                        return additional_property_item_type_16
                    except:  # noqa: E722
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    additional_property_item_type_17 = (
                        PatchTaskRequestAdditionalPropertyItemType17.from_dict(data)
                    )

                    return additional_property_item_type_17

                additional_property_item = _parse_additional_property_item(
                    additional_property_item_data
                )

                additional_property.append(additional_property_item)

            additional_properties[prop_name] = additional_property

        patch_task_request.additional_properties = additional_properties
        return patch_task_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(
        self, key: str
    ) -> List[
        Union[
            "EmbedStep",
            "ErrorWorkflowStep",
            "EvaluateStep",
            "ForeachStep",
            "GetStep",
            "IfElseWorkflowStep",
            "LogStep",
            "ParallelStep",
            "PatchTaskRequestAdditionalPropertyItemType17",
            "PromptStep",
            "ReturnStep",
            "SearchStep",
            "SetStep",
            "SleepStep",
            "SwitchStep",
            "ToolCallStep",
            "WaitForInputStep",
            "YieldStep",
        ]
    ]:
        return self.additional_properties[key]

    def __setitem__(
        self,
        key: str,
        value: List[
            Union[
                "EmbedStep",
                "ErrorWorkflowStep",
                "EvaluateStep",
                "ForeachStep",
                "GetStep",
                "IfElseWorkflowStep",
                "LogStep",
                "ParallelStep",
                "PatchTaskRequestAdditionalPropertyItemType17",
                "PromptStep",
                "ReturnStep",
                "SearchStep",
                "SetStep",
                "SleepStep",
                "SwitchStep",
                "ToolCallStep",
                "WaitForInputStep",
                "YieldStep",
            ]
        ],
    ) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.transition_type import TransitionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.transition_metadata import TransitionMetadata
    from ..models.transition_target import TransitionTarget


T = TypeVar("T", bound="Transition")


@_attrs_define
class Transition:
    """
    Attributes:
        type (TransitionType):
        output (Any):
        created_at (datetime.datetime): When this resource was created as UTC date-time
        updated_at (datetime.datetime): When this resource was updated as UTC date-time
        execution_id (str):
        current (TransitionTarget):
        next_ (TransitionTarget):
        id (str):
        metadata (Union[Unset, TransitionMetadata]):
    """

    type: TransitionType
    output: Any
    created_at: datetime.datetime
    updated_at: datetime.datetime
    execution_id: str
    current: "TransitionTarget"
    next_: "TransitionTarget"
    id: str
    metadata: Union[Unset, "TransitionMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.transition_metadata import TransitionMetadata
        from ..models.transition_target import TransitionTarget

        type = self.type.value

        output = self.output

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        execution_id = self.execution_id

        current = self.current.to_dict()

        next_ = self.next_.to_dict()

        id = self.id

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "output": output,
                "created_at": created_at,
                "updated_at": updated_at,
                "execution_id": execution_id,
                "current": current,
                "next": next_,
                "id": id,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.transition_metadata import TransitionMetadata
        from ..models.transition_target import TransitionTarget

        d = src_dict.copy()
        type = TransitionType(d.pop("type"))

        output = d.pop("output")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        execution_id = d.pop("execution_id")

        current = TransitionTarget.from_dict(d.pop("current"))

        next_ = TransitionTarget.from_dict(d.pop("next"))

        id = d.pop("id")

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, TransitionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = TransitionMetadata.from_dict(_metadata)

        transition = cls(
            type=type,
            output=output,
            created_at=created_at,
            updated_at=updated_at,
            execution_id=execution_id,
            current=current,
            next_=next_,
            id=id,
            metadata=metadata,
        )

        transition.additional_properties = d
        return transition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.entries_chat_ml_role import EntriesChatMLRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_output_chunk_delta_content_type_2_item_type_0 import (
        ChatOutputChunkDeltaContentType2ItemType0,
    )
    from ..models.chat_output_chunk_delta_content_type_2_item_type_1 import (
        ChatOutputChunkDeltaContentType2ItemType1,
    )


T = TypeVar("T", bound="ChatOutputChunkDelta")


@_attrs_define
class ChatOutputChunkDelta:
    """The message generated by the model

    Attributes:
        role (EntriesChatMLRole): ChatML role (system|assistant|user|function_call|function|function_response|auto)
        content (Union[List[Union['ChatOutputChunkDeltaContentType2ItemType0',
            'ChatOutputChunkDeltaContentType2ItemType1']], List[str], str]): The content parts of the message
        name (Union[Unset, str]): Name
        continue_ (Union[Unset, bool]): Whether to continue this message or return a new one
    """

    role: EntriesChatMLRole
    content: Union[
        List[
            Union[
                "ChatOutputChunkDeltaContentType2ItemType0",
                "ChatOutputChunkDeltaContentType2ItemType1",
            ]
        ],
        List[str],
        str,
    ]
    name: Union[Unset, str] = UNSET
    continue_: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_output_chunk_delta_content_type_2_item_type_0 import (
            ChatOutputChunkDeltaContentType2ItemType0,
        )
        from ..models.chat_output_chunk_delta_content_type_2_item_type_1 import (
            ChatOutputChunkDeltaContentType2ItemType1,
        )

        role = self.role.value

        content: Union[List[Dict[str, Any]], List[str], str]
        if isinstance(self.content, list):
            content = self.content

        elif isinstance(self.content, list):
            content = []
            for content_type_2_item_data in self.content:
                content_type_2_item: Dict[str, Any]
                if isinstance(
                    content_type_2_item_data, ChatOutputChunkDeltaContentType2ItemType0
                ):
                    content_type_2_item = content_type_2_item_data.to_dict()
                else:
                    content_type_2_item = content_type_2_item_data.to_dict()

                content.append(content_type_2_item)

        else:
            content = self.content

        name = self.name

        continue_ = self.continue_

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "role": role,
                "content": content,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if continue_ is not UNSET:
            field_dict["continue"] = continue_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_output_chunk_delta_content_type_2_item_type_0 import (
            ChatOutputChunkDeltaContentType2ItemType0,
        )
        from ..models.chat_output_chunk_delta_content_type_2_item_type_1 import (
            ChatOutputChunkDeltaContentType2ItemType1,
        )

        d = src_dict.copy()
        role = EntriesChatMLRole(d.pop("role"))

        def _parse_content(
            data: object,
        ) -> Union[
            List[
                Union[
                    "ChatOutputChunkDeltaContentType2ItemType0",
                    "ChatOutputChunkDeltaContentType2ItemType1",
                ]
            ],
            List[str],
            str,
        ]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_1 = cast(List[str], data)

                return content_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_2 = []
                _content_type_2 = data
                for content_type_2_item_data in _content_type_2:

                    def _parse_content_type_2_item(
                        data: object,
                    ) -> Union[
                        "ChatOutputChunkDeltaContentType2ItemType0",
                        "ChatOutputChunkDeltaContentType2ItemType1",
                    ]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            content_type_2_item_type_0 = (
                                ChatOutputChunkDeltaContentType2ItemType0.from_dict(
                                    data
                                )
                            )

                            return content_type_2_item_type_0
                        except:  # noqa: E722
                            pass
                        if not isinstance(data, dict):
                            raise TypeError()
                        content_type_2_item_type_1 = (
                            ChatOutputChunkDeltaContentType2ItemType1.from_dict(data)
                        )

                        return content_type_2_item_type_1

                    content_type_2_item = _parse_content_type_2_item(
                        content_type_2_item_data
                    )

                    content_type_2.append(content_type_2_item)

                return content_type_2
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    List[
                        Union[
                            "ChatOutputChunkDeltaContentType2ItemType0",
                            "ChatOutputChunkDeltaContentType2ItemType1",
                        ]
                    ],
                    List[str],
                    str,
                ],
                data,
            )

        content = _parse_content(d.pop("content"))

        name = d.pop("name", UNSET)

        continue_ = d.pop("continue", UNSET)

        chat_output_chunk_delta = cls(
            role=role,
            content=content,
            name=name,
            continue_=continue_,
        )

        chat_output_chunk_delta.additional_properties = d
        return chat_output_chunk_delta

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
